use crate::patterns::PixelColor;
use crate::runner::DeviceSequence;
use serde_derive::{Deserialize, Serialize};
use std::collections::HashSet;
use std::fs;


#[derive(Serialize, Deserialize)]
pub struct Config<const N: usize> {
    devices: Vec<Device<N>>,
    colors: Vec<PixelColor>,
}

#[derive(Deserialize, Serialize)]
pub struct Device<const N: usize> {
    pub mac_addresses: Option<HashSet<String>>,
    name: Option<String>,
    pub sequence: DeviceSequence<N>,
}

pub fn load_from_file<const N: usize>(file: &std::path::Path) -> Vec<Device<N>> {
    let file = fs::read_to_string(file).unwrap();
    let config: Config<N> = serde_yaml::from_str(&*file).unwrap();
    config.devices
}
